#!/bin/bash
#
# Copyright (c) 2008-2009 by nexB, Inc. http://www.nexb.com/ - All rights reserved.
# This software is NOT RELEASED YET and NOT LICENSED unless agreed with nexB.
#

if [ "$2" == "" ]
then
  echo
  echo Script to download and extract the DejaCode toolchain
  echo
  echo "Usage: $0 <username> <password> [<proxy>] [<revision>]"
  echo with:
  echo "  <username> your username to the DejaCode Trac instance,"
  echo "  <password> your password to the DejaCode Trac instance,"
  echo "  <proxy>    HTTP proxy to use, in format machine:port, e.g. \"wwwgate0.mot.com:1080\"."
  echo "             If not set, or if set to \"-\", no proxy will be used."
  echo "  <revision> Revision to download."
  echo "             If not set, the latest version will be downloaded."
  echo "              (note that if if <revision> is set, the toolkit is extracted in the current folder)"
  echo
else

  USER=$1
  PASSWORD=$2
  PROXY=$3
  REVISION=$4

  if [ "$PROXY" != "" -a "$PROXY" != "-" ]
  then
     PROXY_COMMAND="-x $PROXY"
  else
     PROXY_COMMAND=
  fi
  
  URL="https://dev.nexb.us/trac/dejacode/changeset/HEAD/trunk/scripts?old_path=%2F&format=zip"

  if [ "$REVISION" == "" ]
  then
    FILENAME=`curl $PROXY_COMMAND -I -k -u $USER:$PASSWORD $URL | grep 'filename=' | sed -e 's/^.*=//g' | sed -e 's/(.zip).*$/.zip/g'` 
    REVISION=`echo $FILENAME | sed -e 's/^.*-//g' | sed -e 's/\..*$//g'`
    SUBFOLDER=scripts-$REVISION
  else
    FILENAME=trunk_scripts-r${REVISION}.zip
    URL=`echo ${URL} | sed -e "s/HEAD/${REVISION}/g"`
    SUBFOLDER=
  fi

  if [ "$FILENAME" == "" ]
  then
    echo 'Failed to download'
  else
    echo "Downloading: $FILENAME from $URL"
    curl $PROXY_COMMAND -k -u $USER:$PASSWORD $URL > $FILENAME


    unzip $FILENAME
    if [ "$SUBFOLDER" == "" ]
    then
       mv trunk/scripts/* .
       rm -rf trunk/scripts
       chmod a+x dpl
    else
       mv trunk/scripts $SUBFOLDER
       chmod a+x $SUBFOLDER/dpl
    fi
  
    rmdir trunk
    echo "DejaCode Toolkit installed in $(pwd)/$SUBFOLDER"
  fi
fi
